/***
* EdFile:  Binary file editor written using TBrowse
*
* Copyright (c) 1990 Nantucket Corp.  All rights reserved.
* 
* Assumes that every function positions the file pointer initially
*
* Compile with /w/n/a
*/

#include "inkey.ch"
#include "setcurs.ch"
#include "fileio.ch"

#include "omenu.ch"

#include "edfile.ch"

#translate EditColorStr(<id>) => ;
            ColorGet(<id>, B_NORM) + "," + ColorGet(<id>, B_NORMHI) + "," + ;
            ColorGet(<id>, B_ACTIVEHI) + "," + ;
            ColorGet(<id>, A_NORM) + "," + ColorGet(<id>, A_NORMHI) + "," + ;
            ColorGet(<id>, A_ACTIVEHI) + "," + ;
            ColorGet(<id>, N_NORM) + "," + ;
            ColorGet(<id>, C_GET)  + "," + ColorGet(<id>, C_SCROLL)

STATIC scRowBuffer
STATIC snBuffBytes

FUNCTION EdFile( cFile )
   LOCAL hBar, hFileMenu
   LOCAL cScreen, cScrTemp
   LOCAL nRow, nCol
   LOCAL lMono
   LOCAL cColorStr
   LOCAL nPID

   IF cFile == NIL
      ?
      ? 'Syntax:'
      ? '  EDFILE <cFileName>'
      ?
      QUIT
   ENDIF

   lMono := !ISCOLOR()

   SAVE SCREEN TO cScreen
   nRow := ROW()
   nCol := COL()

   CreateMenus( @hBar, @hFileMenu )
   CreateDefPalettes()

   nPID := PaletteNew(TB_TOP, 5, TB_BOTTOM, 75)
   IF lMono
      PaletteLoad(nPID, "DEFBW.CLR")
   ELSE
      PaletteLoad(nPID, "DEFCOLOR.CLR")
   ENDIF

   BarDisplay( hBar )
   ClrScr( ColorGet(nPID, C_BACKGRD ) )

   cColorStr := EditColorStr(nPID)

   EditFile( cFile, hBar, cColorStr )

   DO WHILE BarMenuChoice( hBar ) != F_EXIT
      IF BarMenuChoice(hBar) == F_COLOR
         BarDisplay( hBar )
         ClrScr( ColorGet(nPID, C_BACKGRD) )

         ColorSet(nPID, "N/BG")
         IF lMono
            PaletteSave(nPID, "DEFBW.CLR")
         ELSE
            PaletteSave(nPID, "DEFCOLOR.CLR")
         ENDIF
         ClrScr( ColorGet(nPID, C_BACKGRD) )

      ELSE
         DEVPOS( 24,0 )
         ?
         ? "Unexpected return val: " + STR( BarMenuChoice(hBar), 0 )
         ?
         QUIT

      ENDIF

      //... Rebuild color because it may have changed
      cColorStr := EditColorStr(nPID)

      //... Default action is to browse file
      EditFile( cFile, hBar, cColorStr )
   ENDDO

   RESTORE SCREEN FROM cScreen
   DEVPOS(nRow,nCol)

   RETURN (NIL)


/***
* EditFile:  TBrowse edit of file
*/
STATIC FUNCTION EditFile( cFile, hBar, cColorSpec )
   LOCAL oTBBin, oTBAsc
   LOCAL oCol
   LOCAL lCont
   LOCAL nCol := 1
   LOCAL nKey
   LOCAL cTitle
   LOCAL cColorSave := SETCOLOR()
   LOCAL aScroll, aColor

   PRIVATE nHandle
   PRIVATE nBinLine := 1, nAscLine := 1
   PRIVATE i

   IF (nHandle := OpenFile( cFile )) == F_ERROR
      ? 
      ? 'Error opening ' + cFile + ' (' + LTRIM(STR(FERROR())) + ').'
      ?
      QUIT
   ENDIF

   //... Set input to binary side
   Binary(.T.)

   //... Post Exit block so I know how to leave
   PostExitBlock( {|| lCont := .F.} )

   oTBBin := TBrowseNew( TB_TOP, BIN_LEFT, TB_BOTTOM, BIN_RIGHT ) 
   oTBAsc := TBrowseNew( TB_TOP, ASC_LEFT, TB_BOTTOM, ASC_RIGHT )

   //... Movement blocks identical for both
   oTBBin:goTopBlock    := {|| nBinLine := 1 }
   oTBBin:goBottomBlock := {|| nBinLine := LastLine(nHandle)}
   oTBBin:skipBlock     := {|x| BinSkip(nHandle, x, @nBinLine)}

   oTBAsc:goTopBlock    := {|| nAscLine := 1, FillBuffer(nHandle,1,TB_COLUMNS) }
   oTBAsc:goBottomBlock := {|| nAscLine := LastLine(nHandle), FillBuffer(nHandle,nAscLine,TB_COLUMNS)}
   oTBAsc:skipBlock     := {|x| BinSkip(nHandle, x, @nAscLine)}

	/* default heading and column separators */
   oTBBin:headSep := MY_HSEP
   oTBBin:colSep  := MY_CSEP

   oTBAsc:headSep := MY_HSEP
   oTBAsc:colSep  := ''

   aColor := ListAsArray( cColorSpec, "," )
   oTBBin:colorSpec := aColor[EF_BIN]+','+aColor[EF_BINHI]+','+aColor[EF_BINSELHI]+','+;
                       aColor[EF_NORM]
   oTBAsc:colorSpec := aColor[EF_ASC]+','+aColor[EF_ASCHI]+','+aColor[EF_ASCSELHI]+','+;
                       aColor[EF_NORM]

   //...Byte count column
   oCol := TBColumnNew( PADL("File Offset", 15), ;
                        {|| TRANSFORM((nBinLine-1)*10, '999,999,999,999')} )
   oCol:width    := 15
   oCol:defColor := {4,4}
   oTBBin:addColumn(oCol)

   cTitle := SPACE(TB_COLUMNS)
   FOR i := 1 TO TB_COLUMNS
      //...Binary version
      oCol := TBColumnNew( STR(I-1,3,0), ;
                           &("{|x| IIF(x==NIL, BinByte("+str(i,0)+"), SetByte(nHandle,nBinLine,"+str(i,0)+",VAL(x)))}") )
      oCol:width := 3
      IF i == 1
         oCol:headSep  := ''                                                         
         oCol:colSep   := '  '
      ENDIF
      oTBBin:addColumn(oCol)

      //...Ascii version mostly the same, but converts "get byte" differently

      oCol := TBColumnNew( SUBSTR(cTitle,i,1), ;
                           &("{|x| IIF(x==NIL, AscByte("+str(i,0)+"), SetByte(nHandle,nAscLine,"+str(i,0)+",ASC(x)))}") )
      oCol:width := 1
      oTBAsc:addColumn(oCol)

   NEXT

   SET CURSOR OFF

   oTBBin:freeze := 1

   oTBBin:autoLite := .F.
   oTBAsc:autoLite := .F.
   
   //... Draw the separator between columns, make same color as Ascii
   //... column
   SETCOLOR( aColor[EF_ASC] )
   @ TB_TOP+1, BIN_RIGHT+1 TO TB_BOTTOM, BIN_RIGHT+1 DOUBLE
   @ TB_TOP+1, BIN_RIGHT+1 SAY ''

   //... Create scroll bar
   aScroll := ScrollBarNew( TB_TOP, 79, TB_BOTTOM, aColor[EF_SCROLL])
   ScrollBarDisplay(aScroll)
   ScrollBarUpdate(aScroll, nAscLine, LastLine(nHandle), .T.)

   lCont := .T.
   DO WHILE (lCont)
      //... Prevent movement into locked columns
		IF ( oTBBin:colPos <= oTBBin:freeze )
			oTBBin:colPos := oTBBin:freeze + 1
		ENDIF

      //... Stabilize
      DO WHILE !oTBBin:stable .OR. !oTBAsc:stable
         oTBAsc:stabilize()
         oTBBin:stabilize()
      ENDDO

      ScrollBarUpdate( aScroll, nAscLine, LastLine(nHandle) )

      //... Make sure cursor comes up correct color without forcing
      //... redraw of whole screen
      oTBAsc:colorRect( { oTBAsc:rowPos, oTBAsc:colPos, oTBAsc:rowPos, oTBAsc:colPos }, ;
                             IF(Binary(), {1,2}, {1,3}) )
      oTBBin:colorRect( { oTBBin:rowPos, oTBBin:colPos, oTBBin:rowPos, oTBBin:colPos }, ;
                             IF(Binary(), {1,3}, {1,2}) )

      //... Make sure cursor ends up on "active" side
      IF Binary()
         oTBAsc:hilite()
         oTBBin:hilite()
      ELSE
         oTBBin:hilite()
         oTBAsc:hilite()
      ENDIF

      IF oTBAsc:hitBottom .OR. oTBAsc:hitTop
         TONE( 150, 0 )
      ENDIF

      //... Get keystroke
      nKey := INKEY(0)

      //... Handle menu system (lCont will be set to false if the
      //... the menu system wants us to leave)
      BarActivate( hBar, nKey )

      IF lCont
         //... Handle keystroke
         DO CASE
            CASE nKey == K_DOWN
               oTBBin:down()
               oTBAsc:down()

            CASE nKey == K_UP
               oTBBin:up()
               oTBAsc:up()

            CASE nKey == K_PGDN
               oTBBin:pageDown()
               oTBAsc:pageDown()

            CASE nKey == K_PGUP
               oTBBin:pageUp()
               oTBAsc:pageUp()

            CASE nKey == K_CTRL_PGUP
               oTBBin:goTop()
               oTBAsc:goTop()

            CASE nKey == K_CTRL_HOME
               oTBBin:goTop()
               oTBAsc:goTop()

            CASE nKey == K_CTRL_PGDN
               oTBBin:goBottom()
               oTBAsc:goBottom()

            CASE nKey == K_CTRL_END
               oTBBin:goBottom()
               oTBAsc:goBottom()

            CASE nKey == K_LEFT
               //... Key off of Ascii browse because it has no locked
               //... columns
               IF oTBAsc:colPos == 1
                  IF nBinLine == 1
                     //... if on first line, cause beep by trying to move
                     //... up
                     oTBBin:up()
                     oTBAsc:up()
                  ELSE
                     //... if at beginning of line, move to end of previous
                     oTBBin:up()
                     oTBAsc:up()

                     oTBBin:end()
                     oTBAsc:end()
                  ENDIF
               ELSE
                  oTBBin:left()
                  oTBAsc:left()
               ENDIF

            CASE nKey == K_RIGHT
               IF oTBBin:colPos == oTBBin:colCount
                  //... if at end, move to start of next line
                  //... if on last line, just move to start
                  oTBBin:down()
                  oTBAsc:down()

                  oTBBin:home()
                  oTBAsc:home()
               ELSE
                  oTBBin:right()
                  oTBAsc:right()
               ENDIF

            CASE nKey == K_HOME
               oTBBin:home()
               oTBAsc:home()

            CASE nKey == K_END
               oTBBin:end()
               oTBAsc:end()

            CASE nKey == K_TAB
               Binary(!Binary())

            CASE nKey == K_ENTER
               IF oTBAsc:colPos <= snBuffBytes
                  IF Binary()
                     DoGet( oTBBin, "@K 999", {|x| VAL(x) <= 255 .AND. VAL(x) >= 0 }, aColor[8] )
                  ELSE
                     DoGet( oTBAsc, "X", , aColor[8] )
                  ENDIF

                  oTBBin:refreshCurrent()
                  oTBAsc:refreshCurrent()
               ENDIF
            OTHERWISE
               IF (Binary() .AND. ISDIGIT(CHR(nKey))) .OR. !Binary()
                  IF oTBAsc:colPos <= snBuffBytes
                     KEYBOARD CHR(nKey)

                     IF Binary()
                        DoGet( oTBBin, "@K 999", {|x| VAL(x) <= 255 .AND. VAL(x) >= 0 }, aColor[EF_GET] )
                     ELSE
                        DoGet( oTBAsc, "X", , aColor[EF_GET] )
                     ENDIF

                     oTBBin:refreshCurrent()
                     oTBAsc:refreshCurrent()
                  ENDIF
               ENDIF

         ENDCASE

      ENDIF
   ENDDO

   SET CURSOR ON
   SETCOLOR( cColorSave )
   CloseFile( cFile, nHandle )

   //... Remove exit block
   PostExitBlock()

   RETURN NIL


/***
* BinByte() - Must be public, called from a macro
*/
FUNCTION BinByte( nOffset )
   LOCAL cByte

   IF nOffSet > snBuffBytes
      cByte := '   '
   ELSE
      cByte := STR(ASC(SUBSTR( scRowBuffer, nOffset, 1 )),3,0)
   ENDIF

   RETURN (cByte)


/***
* AscByte() - Must be public, called from a macro
*/
FUNCTION AscByte( nOffset )
   LOCAL cByte

   IF nOffSet > snBuffBytes
      cByte := ' '
   ELSE
      cByte := SUBSTR( scRowBuffer, nOffset, 1 )
   ENDIF

   RETURN (cByte)


/***
* SetByte() - Must be public, called from a macro
*/
FUNCTION SetByte( nHandle, nLine, nOffset, nByte )
   LOCAL nLoc

   nLoc := ( (nLine-1) * TB_COLUMNS ) + nOffset-1
   FSEEK( nHandle, nLoc, FS_SET )
   FWRITE( nHandle, CHR(nByte), 1 )

   RETURN (NIL)


/***
* BinSkip()
*/
STATIC FUNCTION BinSkip( nHandle, nMove, pnCurrLine )
   LOCAL nLastLine
   LOCAL nMoved := 0

   nLastLine := LastLine( nHandle )
   IF nMove > 0 
      DO WHILE (pnCurrLine + nMoved < nLastLine) .AND. ( nMoved < nMove )
         nMoved++
      ENDDO
   ELSEIF nMove < 0
      DO WHILE (pnCurrLine + nMoved > 1) .AND. ( nMoved > nMove )
         nMoved--
      ENDDO
   ENDIF

   pnCurrLine += nMoved

   FillBuffer( nHandle, pnCurrLine, TB_COLUMNS )

   RETURN (nMoved)


/***
* FillBuffer()
*/
STATIC FUNCTION FillBuffer( nHandle, nLine, nBytes )

   scRowBuffer := SPACE(nBytes)

   FSEEK( nHandle, (nLine-1)*nBytes, FS_SET )

   snBuffBytes := FREAD( nHandle, @scRowBuffer, nBytes )

   RETURN NIL


/***
* LastLine()
*/
STATIC FUNCTION LastLine( nHandle )
   LOCAL nLastByte

   nLastByte := FSEEK( nHandle, 0, FS_END )

   RETURN( INT(nLastByte / TB_COLUMNS) + IIF((nLastByte % TB_COLUMNS) == 0, 0, 1) )


/***
* OpenFile:  Open the file
*/
STATIC FUNCTION OpenFile( cFile )
   LOCAL nHandle

   nHandle := FOPEN( cFile, FO_READWRITE + FO_EXCLUSIVE )

   //... Initialize buffer
   FillBuffer( nHandle, 1, TB_COLUMNS )

   RETURN nHandle


/***
* CloseFile:  Close the file
*/
STATIC FUNCTION CloseFile( cFile, nHandle )

   FCLOSE( nHandle )

   RETURN NIL


/***
* CreateMenus:  Create bar menus (handles must be passed by reference)
*/
STATIC FUNCTION CreateMenus( phBar, phFile )
   
   phBar  := BarNew()
   phFile := MenuNew('~File')

   PromptAdd( phFile, F_COLOR, 'Co~lor  ' )
   PromptAdd( phFile, F_EXIT,  'E~xit   ESC' )

   MenuAdd( phBar, phFile )

   PromptQuickKey( phBar, phFile, F_EXIT, K_ESC )

   RETURN( NIL )

/***
* CreateDefPalettes()
*/
STATIC FUNCTION CreateDefPalettes
   LOCAL nPID

   //... Create color palette
   IF !FILE("DEFCOLOR.CLR")
      nPID := PaletteNew( TB_TOP, 5, TB_BOTTOM, 75 )

      ColorAdd(nPID, C_BACKGRD, "Background ",                   "W/N")
      ColorAdd(nPID, C_GET,     "Edit Active ",                  "GR+/BG")
      ColorAdd(nPID, C_SCROLL,  "Scroll Bar ",                   "W/N")

      ColorAdd(nPID, N_NORM,    "File Offset ",                  "W/N")

      ColorAdd(nPID, B_NORM,    "Decimal Normal ",               "W/N")
      ColorAdd(nPID, B_NORMHI,  "Decimal Highlight ",            "N/W")
      ColorAdd(nPID, B_ACTIVEHI,"Decimal Active Highlight ",     "W+/R")

      ColorAdd(nPID, A_NORM,    "Ascii Normal ",                 "W/N")
      ColorAdd(nPID, A_NORMHI,  "Ascii Highlight ",              "N/W")
      ColorAdd(nPID, A_ACTIVEHI,"Ascii Active Highlight ",       "W+/R")

      PaletteSave(nPID, "DEFCOLOR" )
   ENDIF

   //... Create monochrome palette
   IF !FILE("DEFBW.CLR")
      nPID := PaletteNew( TB_TOP, 5, TB_BOTTOM, 75 )

      ColorAdd(nPID, C_BACKGRD, "Background ",                   "W/N")
      ColorAdd(nPID, C_GET,     "Edit Active ",                  "N/W*")
      ColorAdd(nPID, C_SCROLL,  "Scroll Bar ",                   "W/N")

      ColorAdd(nPID, N_NORM,    "File Offset ",                  "W/N")

      ColorAdd(nPID, B_NORM,    "Decimal Normal ",               "W/N")
      ColorAdd(nPID, B_NORMHI,  "Decimal Highlight ",            "N/W")
      ColorAdd(nPID, B_ACTIVEHI,"Decimal Active Highlight ",     "W+/N")

      ColorAdd(nPID, A_NORM,    "Ascii Normal ",                 "W/N")
      ColorAdd(nPID, A_NORMHI,  "Ascii Highlight ",              "N/W")
      ColorAdd(nPID, A_ACTIVEHI,"Ascii Active Highlight ",       "W+/N")

      PaletteSave(nPID, "DEFBW" )
   ENDIF

   RETURN (NIL)

/***
*	DoGet()
*/
STATIC FUNCTION DoGet( oBrowse, cPicture, bValid, cColor )
   LOCAL bInsSave, lScoreSave, lExitSave
   LOCAL nKey
   LOCAL oGet, oColumn

	/* save state */
	lScoreSave := SET(_SET_SCOREBOARD, .f.)
	lExitSave  := SET(_SET_EXIT, .t.)
	bInsSave   := SETKEY(K_INS)

	/* set insert key to toggle insert mode and cursor */
	SETKEY( K_INS, ;
		{|| SETCURSOR( IF(READINSERT(!READINSERT()), SC_NORMAL, SC_INSERT))};
		  )

	/* initial cursor setting */
	SETCURSOR( IF(READINSERT(), SC_INSERT, SC_NORMAL) )

   /* get column object */
   oColumn := oBrowse:getColumn(oBrowse:colPos)

	/* create a corresponding GET */
	oGet := GetNew(Row(), Col(), oColumn:block, oColumn:heading, cPicture, "W/N,"+cColor)

   oGet:postBlock := bValid

	/* read it */
	READMODAL( {oGet} )

	/* restore state */
	SETCURSOR(0)
	SET(_SET_SCOREBOARD, lScoreSave)
	SET(_SET_EXIT, lExitSave)
	SETKEY(K_INS, bInsSave)

	/* check exit key */
	nKey := LASTKEY()
	IF ( nKey == K_UP .OR. nKey == K_DOWN .OR. ;
		nKey == K_PGUP .OR. nKey == K_PGDN )

		KEYBOARD( CHR(nKey) )
	ENDIF

   RETURN (NIL)

STATIC FUNCTION Binary( lSet )
   STATIC lBin

   IF lSet != NIL
      lBin := lSet
   ENDIF

   RETURN (lBin)

STATIC FUNCTION ClrScr( cColor )
   LOCAL cColorSave

   cColorSave := SETCOLOR( cColor )
   @ 1,0 CLEAR TO MAXROW(),MAXCOL()
   SETCOLOR( cColorSave )

   RETURN NIL

